# Ivan 'ico' Rako <irako@srce.hr>
# Zoran 'jelly' Dzelajlija <jelly@srce.hr>

notice () {
  LC_MESSAGES=hr_HR dialog --backtitle "$title" --msgbox "$*" 18 75
}

first_warning () {
  LC_MESSAGES=hr_HR dialog --backtitle "$title" --yesno "$msg_first_warning" 18 75
  if [ $? -ne 0 ]; then
    log "Upgrade canceled."
    exit 0
  fi
}

pkg() {
  local version
  dpkg -l "$1" 2> /dev/null | egrep -q "^.i" || return 1
  [ -z "$2" ] && return 0
  version=$(dpkg -s "$1" | awk '/^Version:/ {print $2}')
  dpkg --compare-versions "$version" "$2" "$3" 2>&1 > /dev/null
}

log() {
  local old_umask
  logfile=${logfile:=/var/log/carnet-upgrade.log}

  old_umask=$(umask)
  umask 0077

  echo "$(date +'%Y-%m-%d %H:%M:%S') $*" >> $logfile
  echo "CN: $*"

  umask $old_umask
  chmod og= $logfile
}

# find first free uid/gid in range
# find_id passwd 100 999
find_id() {
  local i db first last ids
  db=$1
  first=$2
  last=$3
  ids=$(getent $db | awk -F: "\$3 >= $first && \$3 <= $last {print \$3}")
  for i in $(seq $first $last)
  do
    if ! echo $ids | grep -q $i; then
      echo $i
      return 0
    fi
  done
  return 1
}

# safe in-place s///
check_and_sed() {
  [ "$DEBIAN_SCRIPT_DEBUG" ] && set -vx
  local s sedcmd ret i
  s="$1"
  shift
  sedcmd="$1"
  shift
  ret=2
  for i in $*
  do
    [ -e "$i" ]        || continue
    egrep -q "$s" "$i" || continue
    [ -h "$i" ]        && i=$(readlink -f "$i")
    sed "$sedcmd" "$i" > "$i.dpkg-tmp"
    chown --reference "$i" "$i.dpkg-tmp"
    chmod --reference "$i" "$i.dpkg-tmp"
    mv "$i.dpkg-tmp" "$i"
    ret=0
  done
  if [ "$ret" -eq 0 ]; then
    log "Done \"$sedcmd\" to $*"
  fi
  return $ret
}

#
# Update uid for user from reserved system range (0-99) to dynamic system
# range (100-999).  Optionally update ownerships of given directories.
# $0 user [directory ...]
#
check_and_update_ugid() {
  local user newgid newuid
  user=$1
  if [ "$(getent passwd $user | awk -F: '$3 >= 100 {print "ok"; exit 0}')" ]; then
    return 0
  fi
  shift
  newgid=$(find_id group 100 999)
  newuid=$(find_id passwd 100 999)
  # other directories/files
  chown -R $newuid:$newgid $*
  groupmod -g $newgid $user
  usermod -u $newuid -g $newgid $user
  log "Fixed $user user uid/gid."
}

pkgadd () {
  [ -n "$*" ] || return 0
  log "Installing $* ..."
  apt-get -y$s install $* || {
    dpkg --configure -a
    apt-get -yf install
    apt-get -y$s install $*
  }
}

pkgrm () {
  [ -n "$*" ] || return 0
  log "Purging $* ..."
  apt-get -y$s --purge remove $* || {
    dpkg --configure -a
    apt-get -yf install
    apt-get -y$s --purge remove $*
  }
}

pkgrm_only () {
  [ -n "$*" ] || return 0
  log "Removing $* ..."
  apt-get -y$s remove $* || {
    dpkg --configure -a
    apt-get -yf install
    apt-get -y$s remove $*
  }
}


apt_update() {
  log "Running apt-get update ..."
  apt-get -y$s update || {
    apt-get -yf install
    apt-get -y$s update
  }
  apt-get autoclean
}

apt_download() {
  log "Running apt-get download ..."
  apt-get -dy$s dist-upgrade
}

dist_upgrade () {
  log "Running dist-upgrade ..."
  apt-get -y$s dist-upgrade || {
    dpkg --configure -a
    apt-get -y$s dist-upgrade
  }
}

remove_x() {
  LC_MESSAGES=hr_HR dialog --backtitle "$title" --yesno "$msg_remove_x" 18 75
  if [ $? -eq 0 ]; then
    log "Removing X Window System"
    pkgrm x11-common
  else
    log "Skipping X Window System removal"
  fi
}

remove_bloat() {
  local to_remove add_this i j n text to_show cmd tmpfile installed
  local update_selections pkgs_to_deselect pkgs_to_remove oldifs pkglist
  text="$msg_remove_bloat"

  [ "$DEBUG" ] && set -x
  n=0
  pkglist=$(mktemp /var/lib/carnet-upgrade/pkglist.XXXXXX)
  COLUMNS=200 dpkg -l | awk '$1 ~ /^.i/ { print $2 }' > $pkglist
  for i in $(echo "$bloats" | sed 's/		*/	/g'| awk -F'	' '{print $2}')
  do
    if grep -q "^$i$" $pkglist; then
      installed="$installed $i"
      add_this="$(echo "$bloats" | egrep "	.*\b${i}\b" | sed 's/		*/	/g'| awk -F'	' '{print $1}')"
      if ! echo "$to_show" | grep -q "$add_this"; then
        to_show="$to_show \"$add_this\" \"\" on"
        n=$(($n+1))
      fi
    fi
  done
  [ "$DEBUG" ] || rm -f $pkglist
  [ -z "$to_show" ] && return 0
  test $n -gt 8 && n=8
  tmpfile=$(mktemp /var/lib/carnet-upgrade/dialog-tmp.XXXXXX)
  cmd="LC_MESSAGES=hr_HR dialog --nocancel --backtitle \""$title"\" --checklist \""$text"\" 20 75 $n $to_show"
  [ "$DEBUG" ] && set +x
    eval $cmd 2> $tmpfile
  [ "$DEBUG" ] && set -x
  to_remove="$(cat $tmpfile | sed 's/\"\ \"/	/g; s/\"\ *//g; s/\\//g')"
  [ "$DEBUG" ] || rm -f $tmpfile 
  [ "$DEBUG" ] && echo "DEBUG: to_remove=$to_remove" 1>&2
  oldifs="$IFS"
  IFS="	" # tab
  for i in $(echo "$to_remove")
  do
    IFS="$oldifs"
    for j in $(echo "$bloats" | grep "^$i" | sed 's/		*/	/g'| awk -F'	' '{print $2}')
    do
      # hopefully this will tell apt not to reinstall them at upgrade time
      update_selections="${update_selections}
        $j	deinstall"
      pkgs_to_deselect="$pkgs_to_deselect $j"
      # only remove those that really are installed
      if echo "$installed" | grep -q "$j"; then
        pkgs_to_remove="$pkgs_to_remove $j"
      fi
    done
  done
  [ "$DEBUG" ] && echo "DEBUG: pkgs_to_remove=$pkgs_to_remove" 1>&2
  # Finally, remove those
  if [ -n "$update_selections" ]; then
    echo "$update_selections" | dpkg --set-selections
    log "Deselected:${pkgs_to_deselect}"
  fi
  eval pkgrm $pkgs_to_remove
}

remove_cn_bloat() {
  local installed pkg cn_pkgs dialog_list selection cn_del to_del del_pkgs

  # get installed packages
  installed=$(mktemp /var/lib/carnet-upgrade/pkglist.XXXXXX)
  COLUMNS=200 dpkg -l | awk '$1 ~ /^.i/ { print $2 }' >$installed

  # detect installed CN packagees
  for pkg in $(echo "$cn_bloats" | cut -d' ' -f1); do
    if grep -qx $pkg $installed; then
      cn_pkgs="$cn_pkgs $pkg"
      dialog_list="$dialog_list $pkg '' off"
    fi
  done
  rm -f $installed

  # report status or finish if no CN packages found
  if [ "$cn_pkgs" ]; then
    log "Found CN packages:$cn_pkgs"
  else
    log "No CN packages found."
    return
  fi

  # user selects the packages to delete
  selection=$(mktemp /var/lib/carnet-upgrade/selection.XXXXXX)
  eval LC_MESSAGES=hr_HR dialog --nocancel --backtitle \""$title"\" \
    --checklist \""$msg_remove_cn_bloat"\" 20 75 6 $dialog_list 2>$selection

  # expand the package list to include the services
  for pkg in $(cat $selection); do
    pkg=${pkg%\"}
    pkg=${pkg#\"}
    cn_del="$cn_del $pkg"
    to_del=$(echo "$cn_bloats" | grep ^$pkg)
    del_pkgs="$del_pkgs $to_del"
  done
  rm -f $selection

  # report selection or finish if no service selected
  if [ "$cn_del" ]; then
    log "Removing CN services:$cn_del"
  else
    log "No CN services selected for removal."
    return
  fi

  # delete the cn packages and relevant services
  pkgrm_only $del_pkgs
}

post_upgrade() {
  touch "$post_upgrade_queuetab"
  if ! grep -q "true	$*" "$post_upgrade_queuetab"; then
    echo "true	$*" >> "$post_upgrade_queuetab"
  fi
}

run_post_upgrade() {
  if [ -f "$post_upgrade_queuetab" ]; then
    run_actions post_upgrade "$post_upgrade_queuetab"
    mv "$post_upgrade_queuetab" "${post_upgrade_queuetab}.done.$(date '+%Y-%m-%d.%H:%M:%S')"
  fi      
}

###

run_actions() {
  local line lineno lineno2 f n name action checkmsg check
  [ "$DEBUG" ] && set -vx
  name="$1"
  f="$2"
  [ -f "$f" ] ||   return 2 # no such file
  n=$(wc -l "$f" | awk '{print $1}')
  [ "$n" -lt 1 ] && return 1 # no lines to process
  log "run_actions ($name) started, processing $n lines from \"$f\"."
  log "-----"
  lineno=0
  lineno2=0
  while [ "$lineno" -lt "$n" ]
  do
    lineno=$(($lineno+1))
    lineno2=$lineno
    line=$(head -$lineno "$f" | tail -1)
    if echo $line | grep -q '^#'; then	# comment
      continue
    fi
    while echo "$line" | egrep -q '\\$'	# continuation
    do
      lineno2=$(($lineno2+1))
      line=$(echo -n "$line"| sed 's/\\$//'; head -$lineno2 "$f" | tail -1)
    done
    check=$(echo "$line" | sed 's/		*/	/g'| awk -F'	' '{print $1}')
    action=$(echo "$line" | sed 's/		*/	/g'| awk -F'	' '{print $2}')
    checkmsg=""
    [ "$check" != "true" ] && checkmsg="if '$check'"
    if eval $check; then
      log "$(printf 'action %02d' $lineno) starting: '${action}' $checkmsg "
      if ! eval $action; then
        log "$(printf 'action %02d' $lineno) failed: '${action}' $checkmsg "
        display_action_failure "$action"
        exit 1
      else  
        log "$(printf 'action %02d' $lineno) finished: '${action}' $checkmsg"
      fi
    else
      log "$(printf 'action %02d' $lineno) skipping: '${action}' $checkmsg"
    fi
    [ "$lineno2" -gt "$lineno" ] && lineno=$lineno2
  done
  log "run_actions ($1) finished"
  log "-----"
}

# report action failure
display_action_failure () {
  local action=$1

  notice "
Doslo je do greske prilikom izvrsavanja akcije '$action'. Detalje problema moguce je vidjeti u log datotekama:

  /var/log/carnet-upgrade.log
  /var/log/carnet-upgrade.typescript

Da bi dovrsili zapocete operacije nad paketima pokrenite:

  # dpkg --configure -a
  # apt-get -f install

Nakon toga ponovno pozovite carnet-upgrade skriptu."
}

# funkcija vraca popis paketa koje treba upgradeati
what_upgrade () {
  local IFSOLD name version package

  for package in `echo "$needed_versions" | awk '{print $1}'| grep -v ^$`; do
    IFSOLD="$IFS"
    IFS="	" # tab
    read name version <<-EOPTS
	$(echo "$needed_versions" | sed 's/		*/	/g' | grep ^$package)
	EOPTS
    IFS="$IFSOLD"
    if pkg "$name" lt "$version"; then
    	echo -n " $name"
    fi
  done
}

# funkcija za ispis paketa koje treba upgradeati
check_upgrade () {
  local upgrade
  upgrade=${1:-"`what_upgrade`"}

  if [ "$upgrade" ]; then
    notice "
Prije prelaska na Debian Lenny potrebno je napraviti nadogradnju ovih paketa:
  $upgrade

Pokrenite:
  # apt-get update
  # apt-get dist-upgrade
ili:
  # apt-get update
  # apt-get install${upgrade}
  
Nakon toga ponovno pozovite ovu skriptu."
    log "carnet-upgrade aborted, upgrade of$upgrade needed."
    exit 1
  fi
}

# update sarge urls in sources list
fix_sarge_sources () {
  local apt_sources='/etc/apt/sources.list'
  local fixed_sources=$(mktemp /var/lib/carnet-upgrade/sources.list.XXXXXX)

  # sarge (oldstable) was archived recently -> sources.list needs update
  awk -v archive='http://archive.debian.org/debian' \
      -v security='http://archive.debian.org/debian-security' \
      '$3 == "sarge" && $2 ~ /[/]debian$/ { $2 = archive }
       $3 == "sarge/updates" && $2 ~ /security\.debian\.org$/ { $2 = security }
       { print }' $apt_sources > $fixed_sources

  # did anything get updated
  if ! cmp $apt_sources $fixed_sources >/dev/null; then
      log "sources.list fixed to use archive.debian.org"

      # backup current version
      if [ ! -f "$apt_sources.$backup_ext.sarge" ]; then
          cp -av $apt_sources "$apt_sources.$backup_ext.sarge"
      fi

      # install fixed version
      cp -av $fixed_sources $apt_sources

      # update the database
      apt_update
  fi

  # delete temporary
  rm -f $fixed_sources
}

# try to check if we are using the latest version of carnet-upgrade script
check_my_version () {
  local packages latest_version

  packages=$(mktemp /var/lib/carnet-upgrade/Packages.XXXXXX)
  if wget -q -O$packages $packages_uri; then
    latest_version=$(sed -ne \
        '/Package: carnet-upgrade/,/Description:/{ /Version: /s///p }' \
        $packages)
    rm -f $packages

    if [ "$latest_version" ]; then
      log "Running carnet-upgrade version is $version"
      log "Latest carnet-upgrade version is $latest_version"
      if dpkg --compare-versions "$version" lt "$latest_version"; then
        check_upgrade " carnet-upgrade"
      else log "Already using the latest carnet-upgrade version."; fi
    else log "Failed to get latest version, skipping check."; fi
  else log "Failed to download Packages file, skipping check."; fi

  rm -f $packages
}

# if php is installed make sure it's version 5
check_php_version () {

  if pkg php4-cn && ! pkg php5-cn; then
    # prepare for php upgrade by installing php5 extensions
    install_php5_extensions

    notice "
Prije prelaska na Debian Lenny potrebno je napraviti nadogradnju sa PHP 4 na PHP 5.

Pokrenite:
  # apt-get update
  # apt-get install php5-cn
  
Nakon sto provjerite da sve web aplikacije rade ispravno, ponovno pozovite ovu skriptu."
    log "carnet-upgrade aborted, upgrade to php5-cn needed."
    exit 1
  fi
}

# install php5 extensions based on installed php4 extensions
install_php5_extensions () {
  local php4_list php5_pkg

  # get a list of installed php4 packages
  php4_list=$(mktemp /var/lib/carnet-upgrade/php4_list.XXXXXX)
  dpkg -l php4-\* | awk '/^ii/ {
    pkg=$2; sub("^php4", "php5", pkg); print pkg }' > $php4_list

  # compare it with the list of available php5 packages
  php5_pkg=$(apt-cache search -n ^php5- | cut -d' ' -f 1 \
    | grep -F -x -f $php4_list | grep -F -x -v php5-cn)

  # handle special cases
  if pkg php4-xslt; then
     php5_pkg="$php5_pkg php5-xsl"
  fi

  # install the required php5 modules
  if [ "$php5_pkg" ]; then
     pkgadd $php5_pkg
  fi

  # cleanup
  rm -f $php4_list
}

# if grsec is installed make sure it is fresh
check_grsec_kernel () {
  local ver=`uname -r`

  log "Kernel version: $ver"
  if [ "$ver" = "${ver%-grsec}" ]; then
    # not grsec variant
    return 0
  fi

  if [ "$ver" = "${ver#2.6.2[4-9]}" ]; then
    # kernel too old
    notice "
Prije prelaska na Debian Lenny potrebno je napraviti nadogradnju na najnoviji kernel.

Pokrenite:
  # apt-get update
  # apt-get install kernel-2.6-cn
  # reboot
  
Nakon toga ponovno pozovite ovu skriptu."
    log "carnet-upgrade aborted, kernel upgrade needed."
    exit 1
  fi
}

check_reboot () {
  reboot_required || return 0

  LC_MESSAGES=hr_HR dialog --backtitle "$title" --yesno "$reboot_warning" 18 +75
  if [ $? -eq 0 ]; then
    log "Reboot before continuing."
    exit 0
  else
    log "Proceed without reboot."
  fi
}

# check if the latest kernel is booted
reboot_required () {
  local default_kernel kernel_package running_release running_version

  # try to get default kernel from lilo
  if type lilo >/dev/null 2>&1; then
      default_kernel=$( lilo -t -v 2>/dev/null \
                        | grep -B3 '[*]$' \
                        | sed -n -e '/^Boot image: / { s///p; q; }' )
  fi

  # fallback to grub if lilo not installed
  if [ -z "$default_kernel" -a -r "/boot/grub/menu.lst" ]; then
      default_kernel=$( awk '
           $1 == "default" { default = $2; 
                             if (default !~ /^[0-9]+$/) { exit } }
           $1 == "title" { title+=1 }
           $1 == "kernel" && title==default+1 { print $2; exit }' \
           /boot/grub/menu.lst 2>/dev/null )
  fi

  if [ -z "$default_kernel" ]; then
      log "Could not find default kernel"
      return 1
  fi

  log "Default kernel image: $default_kernel"

  if [ ! -e "$default_kernel" ]; then
      log "Default kernel image doesn't exist!?"
      return 1
  fi

  kernel_package=$( dpkg -S $default_kernel | cut -d: -f 1 )

  if [ -z "$kernel_package" ]; then
      log "Default kernel not installed from a package"
      return 1
  fi

  log "Default kernel package: $kernel_package"

  running_release=$( uname -r )
  running_version=$( uname -v )
  log "Running kernel: $running_release $running_version"

  if grep -q "$running_release .* $running_version" $default_kernel; then
      log "Latest kernel version is running." 
      return 1
  else
      log "Reboot to latest version required."
      return 0
  fi
}

upgrade () {
  local packages
  case "$1" in
    first)
      packages="$packages_first"
    ;;
    second)
      packages="$packages_second"
    ;;
    *)
    return 1
  esac

  pkgadd_if_installed $packages
}

# install packages introduced in this release
install_new () {
  if [ "$new_packages" ]; then
    pkgadd $new_packages
  fi
}

# funkcija utrpa ispravan sources.list
create_sources_list () {
    local sl sl_new
    sl=/etc/apt/sources.list
    sl_new=/usr/share/carnet-upgrade/files/etc/apt/sources.list

    # check if already installed
    if ! cmp $sl $sl_new >/dev/null; then

        # backup old version
        if [ -f $sl -a ! -f $sl.$backup_ext ]; then
            cp -av $sl $sl.$backup_ext
        fi

        # install new version
        cp -av $sl_new $sl
        notice "Novi sadrzaj datoteke /etc/apt/sources.list:

`cat /etc/apt/sources.list`"
    fi
}


# funkcija koja brise grupu proc
remove_group_proc () {
  if getent group proc > /dev/null; then
    groupdel proc || true
    log "groupdel proc (temporarily)"
    post_upgrade add_group_proc
  fi
}

add_group_proc () {
  if ! getent group proc > /dev/null; then
    # grsec uses gid 99 for /proc files
    groupadd -g 99 proc 
    log "groupadd -g 99 proc"

    # update oidentd so it uses the proc group
    if [ -x /etc/init.d/oidentd ]; then
        if gpasswd -a oident proc; then
            /etc/init.d/oidentd restart
        fi
    fi
  fi
}

# backup ldap database
backup_slapd_db () {
  local ldap_backup=/var/backups/slapcat.$backup_ext.gz
  local old_umask result

  if [ ! -f $ldap_backup ]; then
    [ -x /etc/init.d/freeradius ] && /etc/init.d/freeradius stop || true

    if [ -x /etc/init.d/slapd -a -d /var/lib/ldap ]; then
      /etc/init.d/slapd stop || true
      old_umask=`umask`
      umask 0077 # protect the backup file
      slapcat | gzip --best > $ldap_backup
      result=$?
      umask $old_umask
      /etc/init.d/slapd start || true

      if [ "$result" -eq 0 ]; then
        log "$ldap_backup created"
        notice "Napravljen backup slapd baze u $ldap_backup."
      else
        log "$ldap_backup was not created"
        notice "Backup slapd baze u $ldap_backup nije uspio."
        exit 1
      fi
    fi

    [ -x /etc/init.d/freeradius ] && /etc/init.d/freeradius start || true
  else
    log "slapd backup already at $ldap_backup, skipping."
    #notice "Backup slapd baze u $ldap_backup vec postoji!"
  fi
}

# backup etc direktorija
backup_etc_dir () {
  local etc_backup=/var/backups/etc.$backup_ext.tar.gz
  local old_umask

  if [ ! -f $etc_backup ]; then
    old_umask=`umask`
    umask 0077 # protect the backup file

    if tar cfz $etc_backup /etc 2> /dev/null; then
      log "$etc_backup created"
      notice "Napravljen backup /etc direktorija u $etc_backup."
    else
      log "$etc_backup was not created"
      notice "Backup /etc direktorija u $etc_backup nije uspio."
      exit 1
    fi
    umask $old_umask
  else
    log "/etc backup already present in $etc_backup, skipping."
    #notice "Backup /etc direktorija u $etc_backup vec postoji!"
  fi
}

# fixaj /etc/default/raid2
fix_etc_default_raid2 () {
  if [ -f /etc/default/raid2 ]; then
    sed "s/^AUTOSTART.*/AUTOSTART=true/g" /etc/default/raid2 > /etc/default/raid2.$backup_ext.$$
    chown --reference=/etc/default/raid2 /etc/default/raid2.$backup_ext.$$
    chmod --reference=/etc/default/raid2 /etc/default/raid2.$backup_ext.$$

    if ! cmp -s /etc/default/raid2.$backup_ext.$$ /etc/default/raid2; then
      log "Fixed /etc/default/raid2 AUTOSTART option."
      mv -v /etc/default/raid2.$backup_ext.$$ /etc/default/raid2
    else
      rm -v /etc/default/raid2.$backup_ext.$$
    fi
  fi
}

# restore a configuration file if it contains only CN modifications
restore_file () {
  local file file_expect file_restore file_backup

  file=$1
  file_expect=$2
  file_restore=$3
  file_backup=$file.$backup_ext

  [ "$file_expect" ] || file_expect=$cnup/files/$file.expect
  [ "$file_restore" ] || file_restore=$cnup/files/$file.restore

  # check if all required files are there
  if ! [ -e $file -a -e $file_expect -a -e $file_restore ]; then
    log "Required files missing, skipping $file restore."
    return 1
  fi 

  # is restore needed
  if ! cmp $file_expect $file >/dev/null; then
    log "Config doesn't match the template, skipping $file restore."
    return 1
  fi

  # backup and restore
  if [ ! -e $file_backup ]; then
    log "Restoring config file $file."

    # backup local changes
    cp -av $file $file_backup

    # restore original
    cp -v $file_restore $file

    # restore succedded
    return 0
  else 
    log "Backup file exists, skipping $file restore."
    return 1
  fi
}

# force reconfiguration at the end if package is not upgraded automatically
postupgrade_reconfigure () {
  local cn_package cn_version

  cn_package=$1
  cn_version=$( dpkg -s "$cn_package" | awk '/^Version:/ {print $2}' )
  post_upgrade "pkg $cn_package gt $cn_version || dpkg-reconfigure $cn_package"
}

# copy template into a temporary file
copy_template () {
  local path file template config_new

  path=$1
  file=$(basename $path)
  template=$cnup/files/$path.template
  config_new=$(mktemp /var/lib/carnet-upgrade/$file.XXXXXX)
  cp $template $config_new

  echo $config_new
}

# restore modified config to their package defaults
# so the upgrade doesn't complain so much
restore_configs () {
  local hostname domain config_new memtotal memlimit

  # restore simple configs
  if pkg kernel-2.6-cn && pkg procps lt 1:3.2.7-11; then
     if restore_file /etc/sysctl.conf; then
        postupgrade_reconfigure kernel-2.6-cn
     fi
  fi

  if pkg ntp-cn && pkg ntp lt 1:4.2.4p4+dfsg-8; then
     if restore_file /etc/ntp.conf; then
        postupgrade_reconfigure ntp-cn
     fi
  fi

  if pkg apache2-cn && pkg apache2.2-common lt 2.2.9-10+lenny2; then
     if restore_file /etc/apache2/ports.conf; then
        postupgrade_reconfigure apache2-cn
     fi
  fi

  if pkg postfix-cn && pkg sasl2-bin lt 2.1.22.dfsg1-23; then
     if restore_file /etc/default/saslauthd; then
        postupgrade_reconfigure postfix-cn
     fi
  fi

  if pkg bind9-cn && pkg bind9 lt 1:9.5.1.dfsg.P1-1; then
     if restore_file /etc/bind/named.conf.options; then
        postupgrade_reconfigure bind9-cn
     fi
  fi

  if pkg amavisd-cn && pkg amavisd-new lt 1:2.6.1.dfsg-1; then
     if restore_file /etc/cron.daily/amavisd-new; then
        rm -f /etc/cron.daily/amavisd-new.$backup_ext
        postupgrade_reconfigure amavisd-cn
     fi
  fi

  if pkg spamassassin-cn && pkg razor lt 1:2.85-1; then
     if restore_file /etc/razor/razor-agent.conf; then
        postupgrade_reconfigure spamassassin-cn
     fi
  fi

  if pkg kernel-2.6-cn && pkg libpam-modules lt 1.0.1-5; then
     if restore_file /etc/security/limits.conf; then
        postupgrade_reconfigure kernel-2.6-cn
     fi
  fi

  if pkg samba-cn && pkg samba-common lt 2:3.2.5-4lenny2; then
     if restore_file /etc/samba/smb.conf; then
        postupgrade_reconfigure samba-cn
     fi
  fi

  if pkg base-files lt 5lenny2; then
     restore_file /etc/issue
     restore_file /etc/issue.net
  fi

  # restore php.ini
  memtotal=$(awk '$1 == "MemTotal:" { print int($2/1024) }' /proc/meminfo)
  if [ $memtotal -gt 1024 ]; then
    memlimit=64
  elif [ $memtotal -ge 512 ]; then
    memlimit=32
  else
    memlimit=16
  fi
  
  if pkg php5-cn && pkg php5-cli lt 5.2.6.dfsg.1-1+lenny2; then
     config_new=$(copy_template /etc/php5/cli/php.ini)
     sed -i "s/^[[:space:]]*memory_limit[[:space:]]*=.*/memory_limit = ${memlimit}M/" $config_new
     sed -i "s/^[[:space:]]*post_max_size[[:space:]]*=.*/post_max_size = ${memlimit}M/" $config_new
     if restore_file /etc/php5/cli/php.ini $config_new; then
        postupgrade_reconfigure php5-cn
     fi
     rm -f $config_new
  fi

  if pkg php5-cn && pkg libapache2-mod-php5 lt 5.2.6.dfsg.1-1+lenny2; then
     config_new=$(copy_template /etc/php5/apache2/php.ini)
     sed -i "s/^[[:space:]]*memory_limit[[:space:]]*=.*/memory_limit = ${memlimit}M/" $config_new
     sed -i "s/^[[:space:]]*post_max_size[[:space:]]*=.*/post_max_size = ${memlimit}M/" $config_new
     if restore_file /etc/php5/apache2/php.ini $config_new; then
        postupgrade_reconfigure php5-cn
     fi
     rm -f $config_new
  fi

  # check if monitrc is template based
  if [ -f /etc/monit/monitrc ]; then
     # regenerate config from template
     hostname=$(hostname)
     domain=$(hostname --domain)
     config_new=$(copy_template /etc/monit/monitrc)
     check_and_sed "@localhost" \
       "s/@localhost/@$hostname.$domain/g" $config_new || true
     restore_file /etc/monit/monitrc $config_new
     rm -f $config_new
  fi
}

# temporary disable listchanges packages to reduce clutter during upgrade
apt_listchanges () {
  local file command=$1

  for file in /etc/apt/apt.conf.d/20listchanges \
              /etc/apt/apt.conf.d/20listchanges-cn
  do
      case $command in
           disable)
               if [ -f $file ]; then
                   dpkg-divert --local --rename --divert $file.$backup_ext \
                               --add $file || true
               fi
               ;;

           enable)
               if [ -f $file.$backup_ext ]; then
                   dpkg-divert --remove $file || true
               fi
               ;;
     esac
  done
}

# make a silent installation of carnet and srce keyrings
install_keyrings () {
  pkgadd carnet-keyring srce-keyring debian-archive-keyring
  pkgadd_if_installed dpkg apt debconf python-apt dpkg-dev

  dpkg-reconfigure carnet-keyring srce-keyring debian-archive-keyring
  apt_update
}

# remove skey (not supported anymore)
remove_skey () {
  pkgrm skey-cn libpam-skey

  if getent group skey > /dev/null; then
    groupdel skey || true
    log "groupdel skey"
  fi
}

# remove unsupported php version
# but leave the configuration just in case
remove_php4 () {
  if pkg php5-cn && pkg php4-common; then
     pkgrm_only php4-common
  fi
}

# check if package is orphaned (nothing depends on it)
is_orphaned () {
  local package deps

  package=$1
  deps=$(apt-get remove -s $package | grep ^Remv | wc -l)
  if [ "$deps" -eq 1 ]; then 
    return 0
  else 
    return 1
  fi
}

# remove old and unused libraries
remove_orphaned () {
  local package

  for package in $orphaned_packages; do
    if is_orphaned $package; then
      pkgrm $package
    fi
  done
}

# monit it causing problems for postinst scripts
# restarting daemons so try to disable it
disable_monit () {
  if pkg monit-cn; then
    pkgrm monit-cn # disables monit in prerm
  fi

  # if configured manually
  if pkg monit; then
    # remove the binary but leave the config
    apt-get remove --yes monit
    # stop the binary
    pkill -9 monit || true
  fi
}

# check which of the given packages are installed
grep_installed () {
  local package installed

  for package in $*; do
    if pkg $package; then
      installed="$installed $package"
    fi
  done

  echo $installed
}

# only upgrade packages if they are already installed
pkgadd_if_installed () {
  local packages

  packages=$( grep_installed $* )

  if [ "$packages" ]; then
    eval pkgadd $packages
  fi
}

# make a silent upgrade to new libc6
upgrade_libc () {
  DEBIAN_FRONTEND=noninteractive pkgadd_if_installed libc6 libc6-dev tzdata
}

# upgrade apache2/php5 to lenny
upgrade_apache2 () {
  pkgrm apache-common # prevents installation of apache2-suexec
  pkgadd_if_installed libapache2-mod-php5 php5-cli php5-cn apache2-cn \
    php5-odbc php5-suhosin php-suhosin-cn
}

upgrade_amavis () {
  local conf

  # remove init script diversion before upgrade
  if pkg amavisd-cn lt 3:2.6.1-1; then 
    if [ -L /etc/init.d/amavis -a -f /etc/init.d/amavis.amavisd-new ]; then
      rm -vf /etc/init.d/amavis
      dpkg-divert --remove /etc/init.d/amavis
    fi
  fi

  # move old config aside
  for conf in /etc/amavisd.conf /etc/amavis/amavisd.conf; do
    if [ -e $conf -a ! -e $conf.$backup_ext ]; then
      mv -v $conf $conf.$backup_ext
    fi
  done

  # install new packages
  pkgadd amavisd-cn amavisd-new

  # fix new packages
  check_and_sed '^clamd.*5.clamav.log$' \
                's/^\(clamd.*\)5.clamav.log$/\14\tsocket/g' \
                /etc/init.d/amavisd-cn || true

  # check if link got lost
  if pkg amavisd-cn && [ ! -L /etc/init.d/amavis ]; then
    apt-get install --reinstall --yes amavisd-cn
  fi

  # start new packages
  /etc/init.d/amavis restart
}

# handle freerdius upgrade
upgrade_freeradius () {
  local template config_new password realm hostname basedn

  pkg freeradius-aai lt 2.1.3-0lenny0 || return 0

  # aai team prevents a clean lenny upgrade
  cp -av /etc/freeradius /etc/freeradius.$backup_ext
  pkgrm freeradius-aai freeradius-ldap freeradius
  pkgadd freeradius-aai
}

# prepare for slapd upgrade (caused by freeradius installation)
prepare_openldap () {
  pkg openldap-aai lt 2.4 || return 0

  # openldap-aai expects org.ldif to exists
  local ldif=/var/lib/ldap/org.ldif
  local ldif_backup=/var/lib/carnet-upgrade/org.ldif
  if [ -f $ldif ]; then
    cp -v $ldif $ldif_backup
  fi
}

# fix openldap-aai postinst user handling
upgrade_openldap () {
  pkg openldap-aai lt 2.4 || return 0

  # slapd postinst fails if move_old_database is false
  echo 'slapd slapd/move_old_database boolean true' | debconf-set-selections 
  pkgadd slapd

  # slapd removes org.ldif during switch from ldap -> openldap user
  local ldif=/var/lib/ldap/org.ldif
  local ldif_backup=/var/lib/carnet-upgrade/org.ldif
  if [ -f $ldif_backup -a ! -f $ldif ]; then
    cp -v $ldif_backup $ldif
  fi

  # openldap-aai postinst fails here because of database permissions
  apt-get -y install openldap-aai || true
  chown -R openldap:openldap /var/lib/ldap
  /etc/init.d/slapd restart
  dpkg --configure -a

  # remove ldap user if openldap works
  if getent passwd ldap      > /dev/null && \
     getent passwd openldap  > /dev/null && \
     pgrep -U openldap slapd > /dev/null
  then
     log "userdel ldap (replaced by openldap)"
     userdel ldap || true
  fi
}

# fix postgresql deinstallation during upgrade
upgrade_postgresql () {
  if check_services show 2>&1 | grep -qw postgresql; then
     apt-get install -y postgresql || apt-get -yf install
  fi
}

# upgrade the MTA
upgrade_postfix () {
  pkgadd sasl2-bin postfix-cn

  if pkg postgrey; then
     pkgrm postgrey # not used by postfix-cn anymore
  fi
}

# upgrade bind separately so DNS is not down for too long
# or breaks postinst scripts of other cn packages that depend on
# working resolver
upgrade_bind() {
  pkgadd bind9-cn
}

get_variable () {
  local name=$1 file=$2 val

  if [ -f "$file" ]; then
     val=$(sed -n "/^[[:space:]]*$name[[:space:]]*=/s/[[:space:]]*//gp" $file)
     val=${val#*=}
  fi

  echo $val
}

# restore munin configs for silent upgrade
upgrade_munin () {
  local config backup hostname domain

  for pkg_name in munin munin-node; do
     pkg $pkg_name lt '1.2.6-10~lenny1' || continue

     config=/etc/munin/$pkg_name.conf
     backup=$config.$backup_ext
     if [ -f $config -a ! -f $backup ]; then
        hostname=$(hostname)
        domain=$(hostname --domain)

        cp -av $config $backup
        check_and_sed "$hostname.$domain" \
          "s/$hostname.$domain/localhost.localdomain/g" $config
     fi
     pkgadd $pkg_name
  done

  # force localhost -> hostname replacement in config files
  pkgadd munin-cn
  dpkg-reconfigure munin-cn
}

# udev.preinst fails if devfs configs still exist
upgrade_udev () {
  local config

  for config in /etc/udev/rules.d/devfs.rules \
                /etc/udev/rules.d/compat-full.rules \
                /etc/udev/rules.d/compat.rules
  do
     if [ -e $config ]; then
        rm -v -f $config
     fi
  done

  if [ -f /etc/udev/udev.rules -a ! -f /etc/udev/rules.d/udev.rules ]; then
     ln -sfv ../udev.rules /etc/udev/rules.d/
  fi

  pkgadd udev
}

# fix /etc/dpkg/dpkg.cfg
comment_force_overwrite () {
  if [ -f /etc/dpkg/dpkg.cfg ]; then
    if check_and_sed '^force-overwrite' \
                  's,\(^force-overwrite\),#\1,' \
                  /etc/dpkg/dpkg.cfg; then
      notice "$msg_comment_force_overwrite"
    fi
  fi
}

# try not to overwrite user modified issue files
fix_issue () {
  if cmp /etc/issue $cnup/files/etc/issue.expect.lenny >/dev/null; then
    cp -av /etc/issue /etc/issue.$backup_ext
    cat > /etc/issue <<EOF
Debian GNU/Linux 5.0 (CARNet Debian 5.0) \\n \\l

EOF
  fi

  if cmp /etc/issue.net $cnup/files/etc/issue.net.expect.lenny >/dev/null; then
    cp -av /etc/issue.net /etc/issue.net.$backup_ext
    cat > /etc/issue.net <<EOF
Debian GNU/Linux 5.0 (CARNet Debian 5.0) %h
EOF
  fi
}

check_archives_space() {
  local available_disk_space archives_size
  available_disk_space=$(free_space /var/cache/apt/archives/)
  archives_size=$(download_size dist-upgrade)

  if [ $? -ne 0 ]; then
    log "Failed to get download size, skipping check."
    return 0
  fi

  log "Archives space: ${available_disk_space}MB"
  log "Download size: ${archives_size}MB"

  if [ $available_disk_space -lt $archives_size ]; then
    log "carnet-upgrade aborted: not enough space in /var/cache/apt/archives/ (have ${available_disk_space}MB, need ${archives_size}MB)"
    notice "Premalo slobodnog mjesta u /var/cache/apt/archives/ za skidanje paketa (potrebno je barem ${archives_size}MB). Procitajte poglavlje 'Izvanredne situacije' u dokumentaciji."
    exit 1
  fi
}

check_kernel_space() {
  local available_disk_space kernel_size linux_images pkg ret
  available_disk_space=$(free_space /)
  linux_images=$(apt-cache show kernel-2.6-cn | grep ^Depends: \
    | grep -o 'linux-image[^ ]*')
  for pkg in $linux_images; do
    kernel_size=$(installed_size $pkg)
    ret=$?
    [ $ret -eq 0 ] && break
  done

  if [ $ret -ne 0 ]; then
    log "Failed to get installed size, skipping check."
    return 0
  fi

  log "Free space in /: ${available_disk_space}MB"
  log "Kernel package: $pkg"
  log "Kernel size: ${kernel_size}MB"

  if LC_ALL=C apt-get -s install kernel-2.6-cn \
     | grep -q '^kernel-2.6-cn is already the newest version.$'
  then
     log "Not checking free space in /: kernel-2.6-cn already installed"
     return
  fi

  if [ $available_disk_space -lt $kernel_size ]; then
    log "carnet-upgrade aborted: not enough space in / (have ${available_disk_space}MB, need ${kernel_size}MB)"
    notice "Premalo mjesta u / za instalaciju kernela (potrebno je barem ${kernel_size}MB). Procitajte poglavlje 'Izvanredne situacije' u dokumentaciji."
    exit 1
  fi
}

free_space() {
  df --portability --block-size=1M "$1" | tail -1 | awk '{print $4}'
}

download_size() {
  local size
  size=$(LC_ALL=C apt-get --yes --print-uris "$@" | grep '^Need to get ')

  if [ $? -ne 0 ]; then
    log "Failed to get download size for 'apt-get $@'."
    return 1
  fi

  size=${size#Need to get }
  size=${size%%B*}

  size_to_mb "${size}B"
}

installed_size() {
  local package size
  package=$1
  size=$(apt-cache show $package | grep '^Installed-Size:')

  if [ $? -ne 0 ]; then
    log "Failed to get installed size for '$package'."
    return 1
  fi

  size=${size#*: }

  size_to_mb "${size}kB"
}

size_to_mb() {
  local size
  size=$1
  if [ -z "$size" ]; then
    size=0;
  fi

  if [ "${size%kB}" != "$size" ]; then
    size=$(round ${size%kB})
    size=$(($size/1024+1))
  elif [ "${size%MB}" != "$size" ]; then
    size=$(round ${size%MB})
  elif [ "${size%GB}" != "$size" ]; then
    size=$(round ${size%GB})
    size=$(($size*1024))
  elif [ "${size%[0-9]B}" != "$size" ]; then
    size=$(round ${size%B})
    size=$(($size/1024/1024+1))
  else
    log "unrecognized size unit: $size"
    return 1
  fi

  echo $size
}

round() {
  local num
  num=$1

  if [ "${num%.*}" != "$num" ]; then
    num=${num%.*}
    num=$(($num+1))
  fi

  echo $num
}

# make sure carnet-upgrade doesn't get upgraded until the end
hold_carnet_upgrade() {
  echo carnet-upgrade hold | dpkg --set-selections
}

release_carnet_upgrade() {
  echo carnet-upgrade install | dpkg --set-selections
}
